package provide fading 1.0

##  fading.tcl
##
##  fading.tcl - auto-fading for toplevel windows
##
##  Copyright (c) 2005 <Irrational Numbers> : <aldo.buratti@tiscali.it> 
##
##
##  This program is free software; you can redistibute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation.
##
##  Version 1.0 - ABU - November 2005


# ----  demo:
if 0 {
  # create some toplevel windows
  toplevel .x1
  toplevel .x2
  # add binding to <Enter>, <Leave> events
  bind .x1 <Enter> [list transparency::fadingWindow %W 0.99 +0.1 ]
  bind .x1 <Leave> [list transparency::fadingWindow %W 0.3  -0.1 ]
  bind .x2 <Enter> [list transparency::fadingWindow %W 0.99 +0.1 ]
  bind .x2 <Leave> [list transparency::fadingWindow %W 0.3  -0.1 ]
}


namespace eval transparency {
   namespace export fadingWindow

     # toplevel window $w will be progressively made transparent or opaque
     #  by changing its alpha value.
     # Alpha-value can range from $aMin to $aMax [0.0 .. 1.0] ;
     #  $dv is delta-value : 
     #   $dv>0 ,window will be set more opaque (till $aMax alpha-value)
     #   $dv<0 ,window will be set more transparent (till $aMin alpha-value)
     # 
     # NOTE: never use a full-opaque value (1.0) for $aMax, 
     #  since window-manager will show an ugly blinking effect.
     # If you want a full-opaque window, use 0.99 (no visual difference)
     
   proc fadingWindow { w alphaLimit dv } {
      variable taskID

       # only if window is a toplevel window
      if { $w != [winfo toplevel $w] } return

        # cancel any running progressive fading on the same window (if any)
      if [info exists taskID($w)] {
         after cancel $taskID($w)
      }
       # enforce limits: alphaLimit should be [0.0 .. 1.0]
      if { $alphaLimit < 0.0 } { set alphaLimit 0.0 }
      if { $alphaLimit > 1.0 } { set alphaLimit 1.0 }
      
      _fading $w [wm attributes $w -alpha] $alphaLimit $dv
   }


   proc _fading { w v Limit dv } {
      variable taskID
  
      if { $dv > 0 } {
          if { $v >= $Limit } return
          set v [expr $v + $dv]
          if { $v > $Limit } {
              set v $Limit
          }
      } else {
          if { $v <= $Limit } return
          set v [expr $v + $dv]
          if { $v < $Limit } {
              set v $Limit
          }
      }
       # be careful: maybe window no longer exists !
      if [winfo exists $w] {
          wm attributes $w -alpha $v
           # delayed recursive scheduling 
          set taskID($w) \
              [after 100 [namespace code [list _fading $w $v $Limit $dv]]]
      }
   }

}

# TO DO:
#  the internal array taskID should be cleared ....

