package provide stickyNote 2.0

##  stickynote.tcl
##
##  stickynote.tcl - a sticky note widget.
##
##  The "stickyNote" command creates a new toplevel window looking like a sticky-note.
##  You can then interactively move,resize and change its look:
##    background color
##    border color
##    font and font color
##    transparency (Windows only)
##    always-on-top (Windows only)
##
##  StickyNote has been derived from the following work: 
##    http://mini.net/tcl/stickies
##    by Damon Courtney.
##    Dec-2004
##
##  Copyright (c) 2005 <Irrational Numbers> : <aldo.buratti@tiscali.it> 
##
##
##  This program is free software; you can redistibute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation.
##
##  Version 2.0 - ABU - August 2005



 # WARNING:
 #  The "wm attributes" command should be used only 
 #   for proper platforms.
 #  For other platform this feature should be ignored.
proc is_wm_attributes_enabled {} {
     if {
         [string equal $::tcl_platform(platform) "windows"] 
          &&
         [package vsatisfies $::tk_patchLevel 8.4.8]
        } { 
         return true 
     } else {
         return false
     }
}



package require snit
package require BWidget
 # Note; I prefer using BWidget for "SelectColor" and "SelectFont" .
 # I discarded tk_chooseColor, since it is not modal.
 


snit::widget stickyNote {
   widgetclass StickyNote
   hulltype toplevel

   typevariable default -array {
        borderbg         orange
        background       yellow
        borderwidth      2
        toolbarheight    10
        textfont         {Verdana 10}
        foreground       black
        width            100
        height           150
        alpha            0.9
        minalpha         0.3 
   }
   
    # current sticky
   typevariable current
    # auxiliary var for popup menu
   typevariable current_topmost

    # WARNING:  $stickyMenu is a RESERVED widget name
   typevariable stickyMenu         .__stickyMenu   
    # WARNING:  $stickyChoosePanel is a RESERVED widget name
   typevariable stickyChoosePanel  .__stickyChoosePanel

   typevariable W0
   typevariable H0
   typevariable Xstart
   typevariable Ystart

   typevariable DX
   typevariable DY
   typevariable resizeCursor

   typeconstructor {

     image create photo gripper -data {
        R0lGODlhEAAQAOMAAAAAAIAAAACAAICAAAAAgIAAgACAgMDAwMDcwKbK8ICA
        gNTQyP///wAAAAAAAAAAACH5BAEAAA8ALAAAAAAQABAAAAQm8MlJq70468v2
        ZIr3gOFGemeXpSWnhKcLvxgLW/bYSnnM0zkRJQIAOw==
     }
       
       # define cursor for resizing
     if { [string equal $::tcl_platform(platform) "windows"] } {
         set resizeCursor size_nw_se
     } else {
         set resizeCursor bottom_right_corner
     }

       # pseudo class for resizing
      bind Resizer <ButtonPress-1> \
        [mytypemethod BeginResizeHandler %W %X %Y] 
      bind Resizer <B1-Motion> \
        [mytypemethod ResizeHandler %W %X %Y] 
      bind Resizer <ButtonRelease-1> {
         event generate [winfo parent %W] <<Resized>> 
         event generate [winfo parent %W] <<Modified>> 
      }

     option add *StickyNote.ToolBar.cursor hand2
      bind ToolBar <3>  \
        [mytypemethod PostPopup %W %X %Y]

      bind ToolBar <ButtonPress-1> \
        [mytypemethod BeginMoveHandler %W %X %Y]
      bind ToolBar  <B1-Motion> \
        [mytypemethod MoveHandler %W %X %Y]
      bind ToolBar <ButtonRelease-1> { 
         event generate [winfo parent %W] <<Modified>> 
      }

       # define a popup menu
      $type DefinePopup $stickyMenu 
   }


   typemethod DefinePopup {m} {
     menu $m -tearoff 0

     $m add command -label "Delete Note" \
         -command "destroy \$[mytypevar current]"

     $m add command -label "Clone" \
         -command "\$[mytypevar current] clone"
     $m add separator
     
     $m add command -label "Toolbar Color..." \
         -command "\$[mytypevar current] chooseToolbarColor"
     
     $m add command -label "Sticky Color..." \
         -command "\$[mytypevar current] chooseColor"
     
     $m add command -label "Font..." \
         -command "\$[mytypevar current] chooseFont"    

     $m add command -label "Text Color..." \
         -command "\$[mytypevar current] chooseForeground"    
 
     if { [is_wm_attributes_enabled] } {
         $m add command -label "Transparency..." \
             -command "\$[mytypevar current] chooseTransparency"       
         $m add separator
         $m add checkbutton -label "Always on Top" \
           -variable [mytypevar current_topmost] \
           -command "\$[mytypevar current] toggleOnTop"
     }
   }


   typemethod PostPopup { window X Y } {
     set current [winfo toplevel $window]
     set current_topmost [$current cget -topmost]
     $stickyMenu post $X $Y
   }


   typemethod BeginMoveHandler {toolbarW x y} {   
      set topW [winfo toplevel $toolbarW]
      set DX [expr $x-[winfo x $topW]]
      set DY [expr $y-[winfo y $topW]]
   }

     
   typemethod MoveHandler {toolbarW x y} {
      set topW [winfo toplevel $toolbarW]
      wm geometry $topW +[expr $x-$DX]+[expr $y-$DY]
   }


   typemethod BeginResizeHandler {resizerW x y} {        
     set topW [winfo toplevel $resizerW]
     set H0 [winfo height $topW]
     set W0 [winfo width $topW]
     set Xstart $x
     set Ystart $y
   }

      
   typemethod ResizeHandler {resizerW x y} {
      set topW [winfo toplevel $resizerW]
      set dx [expr $W0+$x-$Xstart]
      set dy [expr $H0+$y-$Ystart]
       # do not resize under 50x50
      if { $dx < 50  ||  $dy < 50 } return
      wm geometry $topW  ${dx}x${dy}
   }

   variable attrChanged false

   option -topmost -default false -configuremethod SetTopMost
   option -alpha  -default 1.0 -configuremethod SetAlpha
   option -text  -default {} -configuremethod SetText -cgetmethod GetText
   option -background -configuremethod SetBg
   option -borderbg -configuremethod SetBorderBg
   
    # options for text-widget, are not delegated *directly*;
    #  they should be intercepted by a local method, then passed to text-widget
   
   # delegate option -state to textW
   # delegate option -font to textW
   # delegate option -foreground to textW
   option -state      -configuremethod SetTextOption -cgetmethod GetTextOption
   option -font       -configuremethod SetTextOption -cgetmethod GetTextOption
   option -foreground -configuremethod SetTextOption -cgetmethod GetTextOption

   delegate method * to textW

   
   method SetTextOption {option value} {
        $textW configure $option $value
        if { ! $attrChanged } {
            set attrChanged true
            event generate $win <<Modified>>
        }
   }
   method GetTextOption {option} {
        return [$textW cget $option]    
   }
   
   method SetBg {option value} {
        set options($option) $value
        $textW configure -bg $value
        $win.gripper configure -bg $value
   }

   method SetText {option value} {
        set options($option) {}  ; # don't waste space
        $textW delete 1.0 end
        $textW insert 1.0 $value
   }
   method GetText {option} {
      return [$textW get 1.0 end]
   }

   method SetAlpha {option value} {
        set options($option) $value
        if { [is_wm_attributes_enabled] } {
	    wm attributes $win -alpha $value
        }
   }

   method SetTopMost {option value} {
        set options($option) $value
        if { [is_wm_attributes_enabled] } {
	    wm attributes $win -topmost $value
        }
   }

   method SetBorderBg {option value} {
        set options($option) $value
        $win.toolbar configure -background $value
        $hull configure -background $value
   }

   
   constructor {args} {      
     wm override $win 1
     
     install toolBarW using frame $win.toolbar -class ToolBar \
        -height $default(toolbarheight)
     pack $win.toolbar -fill x

     install textW using text $win.t -relief flat -font $default(textfont) -wrap word
     label $win.gripper -image gripper -bd 0 -cursor $resizeCursor
     pack $win.t -expand 1 -fill both \
         -padx $default(borderwidth) \
         -pady [list 0 $default(borderwidth)]

     $win configure \
         -background $default(background) \
         -borderbg $default(borderbg)     \
         -foreground $default(foreground)
     
     wm geometry $win ${default(width)}x${default(height)}

     $win configure -alpha $default(alpha)      
     place $win.gripper -in $win.t -relx 1.0 -rely 1.0 -anchor se
     focus $win.t

      # use the bindings of the pseudo-class "Resizer"
     bindtags $win.gripper Resizer
     $win configurelist $args

     # --- ADD traces and binds for trapping changes ----
     # It is important these traps be established here, after
     # the widget has been created and configured, so that
     # we can spare sending events.
     # Only one event is generated at the end of the widget's creation

      # let inner-text events be propagated from StickyNote
      # NOTE: *maybe* you should propagate ALL inner-text events..
     bind $textW <<Modified>> "event generate $win <<Modified>>"

      # every time an option has changed, send an event 
      # ?? should I use mymethod ???
     trace add variable options write [list $win EventGenerate_opt]
     
      # NOTE: it is *important* waiting for the window creation,
      #  or it won't be able to send a message
     tkwait visibility $win
     event generate $win <<Modified>>
   }
     

    # notify an option (attribute)  has changed
   method EventGenerate_opt  {args} {
        if { ! $attrChanged } {
            set attrChanged true
            event generate $win <<Modified>>
        }
   }
   

    # intercepts the edit method, then 'explicetely' re-delegate  ...
   method edit {subCmd args} {
       if { $subCmd == "modified"  &&  $args != {} } {
           set attrChanged $args 
       }
       eval $textW edit $subCmd $args
   }

   method chooseColor {} {
     if [is_wm_attributes_enabled] { 
         after idle [concat wm attributes $stickyChoosePanel -topmost 1]
     }
     set newColor [SelectColor $stickyChoosePanel -title "Sticky color" -color [$win cget -background]]  
     if { $newColor != "" } {
         $win configure -background $newColor
     }
   }
   
   method chooseToolbarColor {} {     
     if [is_wm_attributes_enabled] { 
         after idle [concat wm attributes $stickyChoosePanel -topmost 1]
     }
     set newColor [SelectColor $stickyChoosePanel -title "Toolbar color" -color [$win cget -borderbg]] 
     if { $newColor != "" } {
         $win configure -borderbg $newColor
     }
   }
 
   method chooseForeground {} {
     if [is_wm_attributes_enabled] { 
         after idle [concat wm attributes $stickyChoosePanel -topmost 1]
     }
     set newColor [SelectColor $stickyChoosePanel -title "Text color" -color [$win cget -foreground]] 
     if { $newColor != "" } {
         $win configure -foreground $newColor
     }
   }

   method chooseFont {} {
     if [is_wm_attributes_enabled] { 
         after idle [concat wm attributes $stickyChoosePanel -topmost 1]
     }
     set newFont [SelectFont $stickyChoosePanel -font [$win cget -font]] 
      # Note: '-font' is an undocumented feature
     if { $newFont != "" } {
         $win configure -font $newFont         
     }
   }


   method clone {} {
     set newWin [stickyNote .%AUTO%]
     foreach optionList [$win configure] {
         foreach {opt x y z value} $optionList {break }
         $newWin configure $opt $value
     }
      # clone geometry
     set x [winfo x $win]
     set y [winfo y $win]
     wm geometry $newWin 1x1+[expr $x+10]+[expr $y+10]
     update
     $newWin pumpWindow 1 10 [list [winfo width $win] [winfo height $win]] \
                   { POSITIVE POSITIVE}
     return $newWin
   }
    

    # WARNING:
    # this method should be enabled only for proper platforms
   method chooseTransparency {} {
     set top $stickyChoosePanel

     toplevel     $top
     wm title     $top "Set Transparency"
     wm geometry  $top +[winfo x $win]+[winfo y $win]
     wm resizable $top 0 0
         
     wm attributes $top -topmost 1
     wm attributes $top -toolwindow 1
      # bug fixing:
      # never allow alpha 1.0, or transparency-menu will be covered by
      #  the sticky window when it reaches alpha 1.0.
      # Solution is to limit alpha to 0.99 (no visual difference)
     scale $top.scale -orient horizontal -from $default(minalpha) -to 0.99 \
         -width 10 -length 150 -resolution .01 -showvalue 0 \
         -command [list $win configure -alpha]
     $top.scale set [$win cget -alpha]

     pack  $top.scale -expand 1 -fill both

      # it should be a modal menu. over any top window !!
     grab $top

      # resume control only when this window is destroyed
     tkwait window $top
}
    

    # WARNING:
    # this method should be enabled only for proper platform
   method toggleOnTop {} {
      set val [expr [$win cget -topmost] ? 0 : 1]
      $win configure -topmost $val
   }


    # Basic method for achieving persistency (save/load stickies)
    #  - geometry is not saved -
    # NOTE: since stickyNotes can only hold pure text
    #       (no tags, images, windows...), it's not difficult to serialize
    #       the text contents.
   method serialize {} {
     set cmdStr {}
     set cmdStr "$type create .sticky%AUTO% "
     foreach optionList [$win configure] {
         foreach {opt x y z value} $optionList {break }
         lappend cmdStr $opt $value
     }
     return $cmdStr
   }


    # a way for animating stickyNotes when created (new or cloned)
   method pumpWindow { n N  winSizeList directionList }  {   
       set screenSize(X) [winfo screenwidth $win]
       set screenSize(Y) [winfo screenheight $win]
       
    
       foreach { winSize(X) winSize(Y) }       $winSizeList { break }
       foreach { direction(X) direction(Y) }   $directionList { break }
       
       if { $direction(X) == "POSITIVE" } {
          set fixedPoint(X) [winfo x $win]
       } else {
          set fixedPoint(X) [expr [winfo x $win]+[winfo width $win]]
       }
       if { $direction(Y) == "POSITIVE" } {
          set fixedPoint(Y) [winfo y $win]
       } else {
          set fixedPoint(Y) [expr [winfo y $win]+[winfo height $win]]
       }
    
       foreach axis {X Y} {
           set D($axis) [expr ($n * $winSize($axis))/$N]
           if { $direction($axis) == "POSITIVE" } {
               set R($axis) $fixedPoint($axis)
               if { $R($axis)+$D($axis) > $screenSize($axis) } {
                   set R($axis) [expr $screenSize($axis)-$D($axis)]
               }
           } else {
               set R($axis) [expr $fixedPoint($axis) -$D($axis)]
               if { $R($axis) < 0 } {
                   set R($axis) 0
               }
           }
       } 
       wm geometry $win $D(X)x$D(Y)+$R(X)+$R(Y)
       if { $n < $N } {
          after 20 [mymethod pumpWindow [incr n] $N $winSizeList $directionList]
       }
    }

}
