package provide stickyTpt 2.0


##  stickytpt.tcl
##
##  stickytpt.tcl - a 'template-manager' for stickyNotes
##
##  The "stickyTpt" command creates a new stickyNote acting as a model (template)
##   for the newly created stickyNotes.
##  A stickyTpt is a stickyNote with some special features:
##   * text is read-only
##   * changed properties (i.e. attributes) are copied to the "default" type-variable 
##     of the "stickyNote" command.
##
##  Copyright (c) 2005 <Irrational Numbers> : <aldo.buratti@tiscali.it> 
##
##
##  This program is free software; you can redistibute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation.
##
##  Version 2.0 - ABU - August 2005


package require stickyNote


  # general purpose ...
proc centerToplevel {w} {

   set screenSize(X) [winfo screenwidth $w]
   set screenSize(Y) [winfo screenheight $w]

   set wSize(X) [winfo width $w]
   set wSize(Y) [winfo height $w]
  
   foreach axis {X Y} {
        set P($axis) [expr ($screenSize($axis)-$wSize($axis))/2]
   }
   wm geometry $w +$P(X)+$P(Y)
}


namespace eval stickyTpt {
  ;
}


 #create a (withdrawn) sticky-template.
proc stickyTpt::create {} {
    set w [stickyNote create .tpt_%AUTO%]
    
     # sample-text should read-only.
    $w tag configure center -justify center
    $w insert end "\nSTICKY\nSAMPLE\n" center 
    $w configure -state disabled
    
    DefinePopup $w.menu $w
    
     # stickyTpt's menu should be different from the stickyNote's menu.
      
     # copy all stickyNote's ToolBar bindings and change <3> binding
    foreach ev [bind ToolBar] {
        bind TptToolBar $ev [bind ToolBar $ev]
    }
    bind TptToolBar <3> [list $w.menu post %X %Y]
    
    bindtags $w.toolbar { $w.toolbar TptToolBar $w all}
    
     # whenever the stickyTpt size changes, 
     #  the stickyNote::default(..) should be updated.
    bind $w <<Resized>> { 
        set stickyNote::default(width)  [winfo width %W]
        set stickyNote::default(height) [winfo height %W]       
    }
    
    wm withdraw $w ; update ; centerToplevel $w
    return $w
}


proc stickyTpt::DefinePopup {m w} {
    menu $m -tearoff 0
    
    $m add command -label "Toolbar Color..." \
        -command "stickyTpt::ChooseToolbarColor $w"
    
    $m add command -label "Sticky Color..." \
        -command "stickyTpt::ChooseColor $w"
    
    $m add command -label "Font..." \
        -command "stickyTpt::ChooseFont $w"    
    
    $m add command -label "Text Color..." \
        -command "stickyTpt::ChooseForeground $w"    
    
    if { [is_wm_attributes_enabled] } {
        $m add command -label "Transparency..." \
            -command "stickyTpt::ChooseTransparency $w"       
    }
}


proc stickyTpt::ChooseToolbarColor {w} {
   $w chooseToolbarColor
   set stickyNote::default(borderbg) [$w cget -borderbg]
}


proc stickyTpt::ChooseColor {w} {
   $w chooseColor
   set stickyNote::default(background) [$w cget -background]
}


proc stickyTpt::ChooseFont {w} {
   $w chooseFont
   set stickyNote::default(textfont) [$w cget -font] 
}


proc stickyTpt::ChooseForeground {w} {
   $w chooseForeground
   set stickyNote::default(foreground) [$w cget -foreground]
}


proc stickyTpt::ChooseTransparency {w} {
   $w chooseTransparency
   set stickyNote::default(alpha) [$w cget -alpha]
}

